#!/usr/bin/perl
#
# (C) 2001-2002
#	Domsodi Gergely	<doome@uhulinux.hu>
#	Pozsar Balazs	<pozsy@uhulinux.hu>
#
# This file is originally part of the UHU Linux distribution.
# Distributable under GPL v2.
#

# ----- Pragmatic modules -----------------------------------------------------
use strict;
use warnings;

# -----------------------------------------------------------------------------
package UHU::common;

use POSIX ":sys_wait_h";

use Encode;
use open ':locale';

# ----- Exporter --------------------------------------------------------------
require Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(
debug
_chomp
cat cat1 _put put
nicesize nicetime
get_hashkey
fsize
splitup
sortn

__system _system
__fork _fork

basename dirname __mkdir _mkdir ln pwd
pipein _pipein
_chdir __chdir

pad

check_password
takeElement isElement

target

_wait

reboot
eject
);

# -----------------------------------------------------------------------------
sub debug($) {
	my ($msg) = @_;

	print STDERR $msg."\n" if $ENV{UHU_DEBUG};
}

# -----------------------------------------------------------------------------
sub _chomp {
	$_ and chomp $_ for @_;
	wantarray ? @_ : join '', @_;
}

# -----------------------------------------------------------------------------
sub cat {
	local @ARGV = @_;
	if (-f $_[0]) {
		return (wantarray ? <> : join '', <>);
	} else {
		return 0;
	}
}

sub cat1 {
	local @ARGV = @_;

	if (-f $_[0]) {
		my $str = <>;
		return _chomp $str;
	} else {
		return 0;
	}
}

sub _put($$;$) {
	my ($data, $filename, $utf8) = @_;
	
	if ($ENV{UHU_DRY_RUN}) {
		debug '>[dryrun]> writing to '.$filename;
		debug _chomp $data;
		debug '>[dryrun]> '.$filename.' closed';
	} else {
		debug ">>> writing to $filename";
		if (open(F, '>', $filename)) {
			binmode(F, ':raw:utf8') if $utf8;
			print F _chomp($data)."\n";
			close F;
			debug ">>> $filename closed without errors";
		} else {
			debug ">>> $filename open error! retval: ".
							($? >> 8)." errno: $!";
			debug ">>> pipe open error! retval: ".($? >> 8)." errno: $!"; 
		};
	}
}

sub put($$;$) {
	_put($_[0], $ENV{TARGET}.'/'.$_[1], (defined($_[2]) ? 1 : 0 ));
}

# -----------------------------------------------------------------------------
sub pad($) {
	my ($val) = @_;
	return ($val < 10 ? "0" : "") . $val;
}

sub nicesize($) {
	my ($size) = @_;

	return '0 B' unless $size;
	return (int($size)) . " B" if $size < 1024;
	return (int($size * 10 / 1024**1) / 10) . " KB" if $size < 1024**2;
	return (int($size * 10 / 1024**2) / 10) . " MB" if $size < 1024**3;
	return (int($size * 10 / 1024**3) / 10) . " GB";
}

sub nicetime($) {
	my ($sex) = @_;

	($sex > 24 * 3600) and return 'ezt nem fogod kivrni :-)';
	my $hours = int($sex / 3600);
	$sex -= $hours * 3600;
	my $minutes = int($sex / 60);
	$sex -= $minutes * 60;

	return (pad $hours) . ':' . (pad $minutes) . ':' . (pad $sex);
}

# -----------------------------------------------------------------------------
sub get_hashkey($$) {
	my ($hash, $value) = @_;

	({ reverse %$hash }->{$value});
}

sub takeElement($$) {
	my ($element, $array) = @_;

	[map {($element eq $_) ? () : $_} @{ $array or [] }];
}

sub isElement($$) {
	my ($element, $array) = @_;
	($element eq $_) and return 1 foreach @{ $array or [] };
}

# -----------------------------------------------------------------------------
sub _chroot() {
	($ENV{TARGET} eq '/') ? () : ('chroot', $ENV{TARGET});
}

sub __fork {
	my $pid;

	debug "fork__[@_]";
	unless ($pid = fork) {
		setpgrp 0, 0;
		if ($ENV{LOGFILE}) {
			open STDOUT, ">$ENV{LOGFILE}";
			open STDERR, ">&STDOUT";
		}
		exec @_ or POSIX::_exit(-1);
	}

	$pid;
}

sub _fork {
	__fork(_chroot, @_);
}

sub __system {
	my @command = @_;

	unless ($ENV{UHU_DRY_RUN}) {
		debug ">>> __system [".join('|',@_)."]";
		if (system(@_) == 0) {
			debug ">>> __system [@_] finished without errors"
		 } else {
			debug ">>> __system [@_] finished with errors. retval: ".
				 ($? >> 8);
		 }
		return $?;
	} else {
		debug ">[dryrun]> __system [@_]";
		return 0;
	}
}

sub _system {
	__system(_chroot, @_);
}

sub _pipein($$@) {
	my ($data, $utf8, @command) = @_;
	my $hide;
	/pass/ and $hide = 1 foreach @command;

	if ($ENV{UHU_DRY_RUN}) {
		debug ">[dryrun]> pipeing to [@command]";
		$hide ? debug 'data hidden' :  debug _chomp($data);
		debug ">[dryrun]> pipe closed without errors [@command]";
	} else {
		debug ">>> pipeing to [@command]";
		if (open(F, '|-', @command)) {
			binmode(F, ':raw:utf8') if $utf8;
			print F _chomp($data)."\n";
			$hide ? debug 'data hidden' :  debug _chomp($data);
			close F;
			debug ">>> pipe closed without errors [@command]";
		} else {
			debug ">>> pipe open error! retval: ".($? >> 8)." errno: $!"; 
		};
	}
}

sub pipein($$;$) {
	my($data, $filename, $utf8) = @_;
	$utf8 = (defined($utf8) ? 1 : 0);
	_pipein($data, $utf8, _chroot, $filename);
}

# -----------------------------------------------------------------------------
sub basename($) {
	($_) = @_;

	$_ or return;

	s|/$||;
	s|^.*/||;
	$_ or '/';
}

sub dirname($) {
	($_) = @_;

	$_ or return '.';

	s|/[^/]*$|| or return '.';
	$_ or '.';
}

sub __mkdir($) {
	my ($dir) = @_;

	__system 'mkdir', '-p', $dir;
}

sub _mkdir($) {
	my ($dir) = @_;

	__system 'mkdir', '-p', $ENV{TARGET}.'/'.$dir;
}

sub __chdir($) {
	my ($dir) = @_;

	debug '>>> changing directory to: '.$dir;
	chdir $dir;
}

sub _chdir($) {
	my ($dir) = @_;
	
	__chdir $ENV{TARGET}.'/'.$dir;
}

sub ln($$) {
	my ($orig, $new) = @_;

	_mkdir dirname $new;
	__system 'ln', '-s', $orig, $new;
}

sub _wait() {
	waitpid -1, &WNOHANG;
}

sub check_password($$) {
	my ($pw, $pw2) = @_;

	my $error = '';

	unless ($pw =~ /.{6}/) {
		$error .= _("Error: The password must be at least 6 characters long!\n");
	}

	if ($pw ne $pw2) {
		$error .= _("Error: The entered passwords do not match!\n");
	}
	
	foreach (split //,$pw) {
		if (ord > 127 ) {
			$error .= _("Error: The password should not contain accented letters!\n");
			last;
		}
	}
	$error;
}

sub reboot() {
	unless ($ENV{UHU_DRY_RUN}) { 
		require 'syscall.ph';
		syscall(&SYS_reboot, 0xfee1dead, 672274793, 0x01234567);
	}
}

sub halt() {
	unless ($ENV{UHU_DRY_RUN}) { 
		require 'syscall.ph';
		syscall(&SYS_reboot, 0xfee1dead, 672274793, 0xcdef0123);
	}
}

sub poweroff() {
	unless ($ENV{UHU_DRY_RUN}) { 
		require 'syscall.ph';
		syscall(&SYS_reboot, 0xfee1dead, 672274793, 0x4321fedc);
	}
}

sub eject($) {
	my ($device) = @_;

	unless ($ENV{UHU_DRY_RUN}) {
		require Fcntl;
		sysopen(CDROM, $device, &Fcntl::O_RDONLY|&Fcntl::O_NONBLOCK);
		ioctl(CDROM, 0x5309, 0); # 0x5309 is CDROMEJECT
		close CDROM;
	}
}

# -----------------------------------------------------------------------------
sub target() {
	$ENV{TARGET};
}

# ----- Initialization --------------------------------------------------------
$ENV{TARGET} ||= '/';

# ----- Beatiful perl :) ------------------------------------------------------
"Ide valami viccet kene irni, nem?";
